@@ -9,6 +9,11 @@ module RSpec
         end
       end
 
+      unless defined?(Mutex)
+        Support.require_rspec_support 'mutex'
+        Mutex = Support::Mutex
+      end
+
       # @private
       def ensure_implemented(*_args)
         # noop for basic proxies, see VerifyingProxy for behaviour.
@@ -16,15 +21,32 @@ module RSpec
 
       # @private
       def initialize(object, order_group, options={})
+        ensure_can_be_proxied!(object)
+
         @object = object
         @order_group = order_group
         @error_generator = ErrorGenerator.new(object)
         @messages_received = []
+        @messages_received_mutex = Mutex.new
         @options = options
         @null_object = false
         @method_doubles = Hash.new { |h, k| h[k] = MethodDouble.new(@object, k, self) }
       end
 
+      # @private
+      def ensure_can_be_proxied!(object)
+        return unless object.is_a?(Symbol) || object.frozen?
+        return if object.nil?
+
+        msg = "Cannot proxy frozen objects"
+        if Symbol === object
+          msg << ". Symbols such as #{object} cannot be mocked or stubbed."
+        else
+          msg << ", rspec-mocks relies on proxies for method stubbing and expectations."
+        end
+        raise ArgumentError, msg
+      end
+
       # @private
       attr_reader :object
 
@@ -90,27 +112,31 @@ module RSpec
           @error_generator.raise_expectation_on_unstubbed_method(expected_method_name)
         end
 
-        @messages_received.each do |(actual_method_name, args, received_block)|
-          next unless expectation.matches?(actual_method_name, *args)
+        @messages_received_mutex.synchronize do
+          @messages_received.each do |(actual_method_name, args, received_block)|
+            next unless expectation.matches?(actual_method_name, *args)
 
-          expectation.safe_invoke(nil)
-          block.call(*args, &received_block) if block
+            expectation.safe_invoke(nil)
+            block.call(*args, &received_block) if block
+          end
         end
       end
 
       # @private
       def check_for_unexpected_arguments(expectation)
-        return if @messages_received.empty?
+        @messages_received_mutex.synchronize do
+          return if @messages_received.empty?
 
-        return if @messages_received.any? { |method_name, args, _| expectation.matches?(method_name, *args) }
+          return if @messages_received.any? { |method_name, args, _| expectation.matches?(method_name, *args) }
 
-        name_but_not_args, others = @messages_received.partition do |(method_name, args, _)|
-          expectation.matches_name_but_not_args(method_name, *args)
+          name_but_not_args, others = @messages_received.partition do |(method_name, args, _)|
+            expectation.matches_name_but_not_args(method_name, *args)
+          end
+
+          return if name_but_not_args.empty? && !others.empty?
+
+          expectation.raise_unexpected_message_args_error(name_but_not_args.map { |args| args[1] })
         end
-
-        return if name_but_not_args.empty? && !others.empty?
-
-        expectation.raise_unexpected_message_args_error(name_but_not_args.map { |args| args[1] })
       end
 
       # @private
@@ -141,17 +167,23 @@ module RSpec
 
       # @private
       def reset
-        @messages_received.clear
+        @messages_received_mutex.synchronize do
+          @messages_received.clear
+        end
       end
 
       # @private
       def received_message?(method_name, *args, &block)
-        @messages_received.any? { |array| array == [method_name, args, block] }
+        @messages_received_mutex.synchronize do
+          @messages_received.any? { |array| array == [method_name, args, block] }
+        end
       end
 
       # @private
       def messages_arg_list
-        @messages_received.map { |_, args, _| args }
+        @messages_received_mutex.synchronize do
+          @messages_received.map { |_, args, _| args }
+        end
       end
 
       # @private
@@ -162,7 +194,9 @@ module RSpec
       # @private
       def record_message_received(message, *args, &block)
         @order_group.invoked SpecificMessage.new(object, message, args)
-        @messages_received << [message, args, block]
+        @messages_received_mutex.synchronize do
+          @messages_received << [message, args, block]
+        end
       end
 
       # @private
@@ -196,6 +230,7 @@ module RSpec
           @object.__send__(:method_missing, message, *args, &block)
         end
       end
+      ruby2_keywords :message_received if respond_to?(:ruby2_keywords, true)
 
       # @private
       def raise_unexpected_message_error(method_name, args)
@@ -245,12 +280,14 @@ module RSpec
           expectation.matches?(method_name, *args)
         end
       end
+      ruby2_keywords :find_matching_expectation if respond_to?(:ruby2_keywords, true)
 
       def find_almost_matching_expectation(method_name, *args)
         find_best_matching_expectation_for(method_name) do |expectation|
           expectation.matches_name_but_not_args(method_name, *args)
         end
       end
+      ruby2_keywords :find_almost_matching_expectation if respond_to?(:ruby2_keywords, true)
 
       def find_best_matching_expectation_for(method_name)
         first_match = nil
@@ -267,10 +304,12 @@ module RSpec
       def find_matching_method_stub(method_name, *args)
         method_double_for(method_name).stubs.find { |stub| stub.matches?(method_name, *args) }
       end
+      ruby2_keywords :find_matching_method_stub if respond_to?(:ruby2_keywords, true)
 
       def find_almost_matching_stub(method_name, *args)
         method_double_for(method_name).stubs.find { |stub| stub.matches_name_but_not_args(method_name, *args) }
       end
+      ruby2_keywords :find_almost_matching_stub if respond_to?(:ruby2_keywords, true)
     end
 
     # @private
@@ -326,6 +365,7 @@ module RSpec
         end
         super
       end
+      ruby2_keywords :message_received if respond_to?(:ruby2_keywords, true)
 
     private
 
